"use client";

import DataTable from "@/components/tables/DataTable";
import { columnsDataBlocks } from "../variables/columnsBlock";
import { TableData } from "@/variables/columnsData";
import { Button, Flex, FormControl, FormLabel, Icon, Modal, ModalBody, ModalCloseButton, ModalContent, ModalFooter, ModalHeader, ModalOverlay, Radio, RadioGroup, Stack, Text, useColorModeValue, useDisclosure, useToast } from "@chakra-ui/react";
import { DeleteIcon, EditIcon } from "@chakra-ui/icons";
import FormComponent from "@/components/form/Form";
import { InputWrapper, RadioGroupWrapper } from "@/components/fields/Fields";
import { useEffect, useMemo, useRef, useState } from "react";
import prepareValidationSchema, { FormObject } from "@/utils/validation-schema";
import { useRouter } from "next/navigation";
import { addClub, editClub } from "@/app/lib/club/action";
import { MyClubInterface } from "../type/my-club";
import { useGeneral } from "@/contexts/GeneralContext";
import { RiCoinFill } from "react-icons/ri";

interface MyClubTableProps {
  data: MyClubInterface[]; // This is required
}

const addFormFields: FormObject[] = [
  {
    name: "name",
    label: "Club Name",
    type: "text",
    defaultValue: "",
    required: true,
  },
];

const editFormFields: FormObject[] = [
  {
    name: "name",
    label: "Club Name",
    type: "text",
    defaultValue: "",
    required: true,
  },
];

interface formValues {
  name: string;
  coins?: string;
}

const MyClubTable: React.FC<MyClubTableProps> = ({ data }: { data: MyClubInterface[] }) => {
  // Chakra imports
  const { isOpen, onOpen, onClose } = useDisclosure();
  const { currentUser } = useGeneral();
  const toast = useToast();
  const router = useRouter();

  // Style const
  const brandColor = useColorModeValue("brand.500", "white");
  const brandStars = useColorModeValue("brand.500", "brand.400");
  const textColor = useColorModeValue("navy.700.700", "white");
  const coinColor = useColorModeValue("yellow.500", "yellow.400");

  // logic values
  const formRef = useRef(null);
  const [dynamicFormField, setDynamicFormField] = useState(addFormFields);
  const formDetails = useMemo(() => prepareValidationSchema(dynamicFormField), [dynamicFormField]);
  const [clubForm, setClubForm] = useState({});
  const [isEdit, setIsEdit] = useState<Boolean>(false);
  const [currentClub, setCurrentClub] = useState<MyClubInterface>({
    name: "",
    coins: "",
  });
  const { initialFormData, validationSchema } = formDetails;
  const [formValidationSchema, setFormValidationSchema] = useState({});
  const modalText = isEdit ? "Edit Club" : "Add Club";

  useEffect(() => {
    setClubForm(initialFormData);
    setFormValidationSchema(validationSchema);
  }, [initialFormData, validationSchema, dynamicFormField]);

  const handleAddClick = () => {
    setIsEdit(false);
    setDynamicFormField(() => {
     return addFormFields
    });
    setCurrentClub(() => {
      return {
        name: "",
        coins: "",
      };
    });
    onOpen()
  }

  // function constant here
  const handleEditClick = async (id: number) => {
    setIsEdit(true);
    setDynamicFormField((prev: any) => {
      prev[0] = {
        name: "name",
        label: "Name",
        type: "text",
        defaultValue: data[id].name,
        required: true,
      };
      prev[1] = {
        name: "coins",
        label: "Coins",
        type: "text",
        defaultValue: data[id].coins,
        required: true,
      };
      return [...prev];
    });
    setCurrentClub((prev) => {
      return { ...data[id] };
    });
    onOpen();
  };

  const handleSubmitClick = () => {
    formRef?.current?.externalHandleSubmit();
  };

  const handleSubmit = async (values: formValues, data: any) => {
    if (isEdit) {
      const availableCoin = currentUser.coins;
      if (parseInt(values.coins) > availableCoin) {
        toast({
          title: "301",
          description: "Available coins is insufficient",
          status: "error",
          position: "top-right",
          duration: 2000,
          isClosable: true,
        });
        return false;
      }
      const editPayload = {
        name: values.name,
        coins: values.coins,
      };
      const res = await editClub(editPayload, currentClub._id);
      if (res.status || res.status == 200) {
        toast({
          title: "200",
          description: res?.message,
          status: "success",
          position: "top-right",
          duration: 2000,
          isClosable: true,
        });
        onClose();
        router.refresh();
        return false;
      }
      toast({
        title: res.status,
        description: res.error,
        status: "error",
        position: "top-right",
        duration: 9000,
        isClosable: true,
      });
    } else {
      const addPayload = {
        name: values.name,
      };
      const res = await addClub(addPayload);
      if (res.status || res.status == 200) {
        toast({
          title: "200",
          description: res?.message,
          status: "success",
          position: "top-right",
          duration: 2000,
          isClosable: true,
        });
        onClose();
        router.refresh();
        return false;
      }
      toast({
        title: res.status,
        description: res.error,
        status: "error",
        position: "top-right",
        duration: 9000,
        isClosable: true,
      });
    }
    // onClose();
    // router.refresh();
    // const res = await editClub(payload, currentClub._id);
    // 
    // toast({
    //   title: "200",
    //   description: "coins request transfer successfully",
    //   status: "success",
    //   position: "top-right",
    //   duration: 2000,
    //   isClosable: true,
    // });
  };

  const action = (id: number) => (
    <Flex gap={2}>
      <Button
        variant="blank"
        onClick={() => {
          handleEditClick(id);
        }}
        alignItems="center"
        justifyContent="center"
        w="37px"
        h="37px"
        lineHeight="100%"
        borderRadius="10px">
        <Icon as={EditIcon} color={brandColor} w="20px" h="20px" />
      </Button>
    </Flex>
  );

  const onChange = (newValue: string) => {};

  return (
    <>
      <Flex justifyContent="end" pb={2}>
        <Button variant="brand" onClick={handleAddClick} type="submit" mr={3}>
          Add
        </Button>
      </Flex>
      <DataTable columnsData={columnsDataBlocks} tableData={data as unknown as TableData[]} isSelectable={false} showPageSize={false} actionSlot={action} />

      <Modal onClose={onClose} size="lg" isOpen={isOpen}>
        <ModalOverlay />
        <ModalContent bg={"navy.900"}>
          <ModalHeader display="flex" alignItems="center" gap="2">
            {modalText}
            <Text as="span" display="flex" alignItems="center" gap={1}>
              ( <Icon as={RiCoinFill} color={coinColor} w="20px" h="20px" />
              <Text color={textColor} fontSize="xl" fontWeight="700" lineHeight="100%">
                {currentUser.coins.toFixed(2)}
              </Text>
              )
            </Text>
          </ModalHeader>
          <ModalCloseButton />
          <ModalBody>
            <FormComponent
              ref={formRef}
              validationSchema={formValidationSchema}
              initialValues={clubForm}
              submitForm={(formValues, formData) => {
                handleSubmit(formValues, formData);
              }}>
              <FormControl w="full">
                <FormLabel display="flex" ms="4px" fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                  Name<Text color={brandStars}>*</Text>
                </FormLabel>
                <InputWrapper form={true} fieldName="name" defaultValue={"" + currentClub.name} id="name" placeholder="club name" variant="auth" w="full" mb="2px" fontSize="sm" ms={{ base: "0px", md: "0px" }} type="text" fontWeight="500" />
                {isEdit && (
                  <>
                    <FormLabel display="flex" ms="4px" fontSize="sm" fontWeight="500" color={textColor} mb="8px">
                      Coins<Text color={brandStars}>*</Text>
                    </FormLabel>
                    <InputWrapper form={true} fieldName="coins" defaultValue={"" + currentClub.coins} id="coins" placeholder="0" variant="auth" w="full" mb="2px" fontSize="sm" ms={{ base: "0px", md: "0px" }} type="number" fontWeight="500" />
                  </>
                )}
              </FormControl>
            </FormComponent>
          </ModalBody>
          <ModalFooter>
            <Button variant="brand" onClick={handleSubmitClick} type="submit" mr={3}>
              Save
            </Button>
            <Button onClick={onClose}>Cancel</Button>
          </ModalFooter>
        </ModalContent>
      </Modal>
    </>
  );
};

export default MyClubTable;
